@@ -1,6 +1,5 @@
 import {Typography} from '@mui/material';
 import Chip from '@mui/material/Chip';
-import ListSubheader from '@mui/material/ListSubheader';
 import {styled, useTheme} from '@mui/material/styles';
 import TextField from '@mui/material/TextField';
 import useMediaQuery from '@mui/material/useMediaQuery';
@@ -10,6 +9,7 @@ import React from 'react';
 import {sortableContainer, sortableElement} from 'react-sortable-hoc';
 import {VariableSizeList} from 'react-window';
 import Popper from '@mui/material/Popper';
+import Link from '@mui/material/Link';
 
 
 const LISTBOX_PADDING = 0; // px
@@ -38,13 +38,24 @@ function renderRow(props) {
 
 
     if (dataSet.hasOwnProperty('group')) {
+        if (dataSet.group === '') {
+            return null;
+        }
         inlineStyle.whiteSpace = 'nowrap';
-        return (
-            <ListSubheader disableGutters={true} key={dataSet.key} component="div" style={inlineStyle}>
-                {dataSet.group}
-            </ListSubheader>
-        );
+        const textMatch = dataSet.inputValue != null ? getTextMatch(dataSet.group, dataSet.inputValue) : null;
+        if (textMatch) {
+            return <Typography component="li" {...dataSet[0]} style={inlineStyle}>
+                {textMatch[0]}<b>{textMatch[1]}</b>{textMatch[2]} {dataSet.selectGroup && <Link href="#"
+                                                                                                onClick={e => dataSet.selectGroup(e, dataSet.group)}>All</Link>}
+            </Typography>;
+        }
+        return <Typography component="li" {...dataSet[0]} style={inlineStyle}>
+            {dataSet.group} {dataSet.selectGroup &&
+            <Link href="#" onClick={e => dataSet.selectGroup(e, dataSet.group)}>All</Link>}
+        </Typography>;
+
     }
+
     const item = dataSet[1];
     const text = item.text != null ? item.text : item;
     const icon = item.icon != null ? item.icon : null;
@@ -81,10 +92,13 @@ function useResetCache(data) {
 // Adapter for react-window
 const ListboxComponent = React.forwardRef(function ListboxComponent(props, ref) {
     const {children, ...other} = props;
-    const itemData = [];
+    let itemData = [];
+
     children.forEach((item) => {
         itemData.push(item);
-        itemData.push(...(item.children || []));
+        if (item.children) {
+            itemData = itemData.concat(item.children);
+        }
     });
 
     const theme = useTheme();
@@ -93,11 +107,11 @@ const ListboxComponent = React.forwardRef(function ListboxComponent(props, ref)
     });
 
     const itemCount = itemData.length;
-    const itemSize = smUp ? 24 : 36;
+    const itemSize = smUp ? 24 : 30;
 
     const getChildSize = (child) => {
         if (child.hasOwnProperty('group')) {
-            return 36;
+            return child.group === '' ? 0 : 30;
         }
 
         return itemSize;
@@ -150,13 +164,14 @@ const StyledPopper = styled(Popper)({
 
 export default function AutocompleteVirtualized(props) {
     const ref = React.createRef();
+    const inputValueRef = React.createRef('');
+    const {value, options, onChange} = props;
 
     function onDrop(event) {
         event.preventDefault();
         event.stopPropagation();
-        let dt = event.dataTransfer;
-        let files = dt.files;
-        let reader = new FileReader();
+        const files = event.dataTransfer.files;
+        const reader = new FileReader();
         reader.onload = function (event) {
             let text = event.target.result.trim();
             let tokens = text.split(/[\n,\t]/);
@@ -169,15 +184,15 @@ export default function AutocompleteVirtualized(props) {
 
         reader.readAsText(files[0]);
         showDragIndicator(false);
-    };
+    }
 
     function enterTokens(event, tokens) {
-        const results = props.value;
-        tokens = tokens.map(token => token.toLowerCase().trim().replace(/"/g, '')).filter(token => token !== '');
+        const results = value;
+        tokens = tokens.map(token => token.toLowerCase().trim().replace(/["']/g, '')).filter(token => token !== '');
         if (tokens.length > 0) {
             let textToOption = new Map();
-            for (let i = 0; i < props.options.length; i++) {
-                const option = props.options[i];
+            for (let i = 0; i < options.length; i++) {
+                const option = options[i];
                 const text = option.text != null ? option.text : option;
                 const textLowerCase = text.toLowerCase();
                 textToOption.set(textLowerCase, option);
@@ -194,7 +209,7 @@ export default function AutocompleteVirtualized(props) {
                 }
             });
         }
-        props.onChange(event, results);
+        onChange(event, results);
     }
 
     function onPaste(event) {
@@ -203,29 +218,28 @@ export default function AutocompleteVirtualized(props) {
             event.preventDefault();
             event.stopPropagation();
             text = text.trim();
-            let tokens = text.split(/[\n,\t]/);
-            enterTokens(event, tokens);
+            enterTokens(event, text.split(/[\n,\t]/));
         }
-    };
+    }
 
     function showDragIndicator(show) {
         ref.current.style.background = show ? '#1976d2' : '';
     }
 
     function onDragOver(event) {
-        if (event.dataTransfer.files.length > 0) {
+        if (event.dataTransfer.items.length > 0) {
             event.preventDefault();
             event.stopPropagation();
             showDragIndicator(true);
         }
 
-    };
+    }
 
     function onDragEnd(event) {
         event.preventDefault();
         event.stopPropagation();
         showDragIndicator(false);
-    };
+    }
 
 
     const onChipClick = props.onChipClick ? (event, option) => {
@@ -255,6 +269,17 @@ export default function AutocompleteVirtualized(props) {
         getOptionLabel = props.groupBy ? (option) => option.text : (option) => option;
     }
 
+    const selectGroup = props.selectGroup ? (event, group) => {
+        const uniqueValues = new Set(value);
+        options.forEach(option => {
+            if (option.group === group && !uniqueValues.has(option.id)) {
+                uniqueValues.add(option.id);
+            }
+        });
+        // ensure everything in group is added
+        // newValue.splice(index, 1);
+        props.onChange(event, Array.from(uniqueValues));
+    } : null;
 
     const filterOptions = (options, {inputValue}) => {
         inputValue = inputValue.trim().toLowerCase();
@@ -277,22 +302,27 @@ export default function AutocompleteVirtualized(props) {
                 }
             } else if (index !== -1) {
                 containsMatches.push(option);
+            } else if (props.groupBy) { // check group
+                const groupIndex = option.group.toLowerCase().indexOf(inputValue);
+                if (groupIndex !== -1) {
+                    exactMatches.push(option);
+                }
             }
         }
         return exactMatches.concat(startsWithMatches).concat(containsMatches);
     };
     const handleTagDelete = (event, index) => {
-        const newValue = props.value.slice();
+        const newValue = value.slice();
         newValue.splice(index, 1);
         props.onChange(event, newValue);
     };
 
     const onSortEnd = (event) => {
-        const newValue = props.value.slice();
+        const newValue = value.slice();
         const tmp = newValue[event.newIndex];
         newValue[event.newIndex] = newValue[event.oldIndex];
         newValue[event.oldIndex] = tmp;
-        props.onChange(event, newValue);
+        onChange(event, newValue);
     };
 
     const SortableItem = sortableElement(({option, sortIndex}) => {
@@ -300,7 +330,7 @@ export default function AutocompleteVirtualized(props) {
             <Chip
                 tabIndex="-1"
                 key={sortIndex}
-                style={{zIndex: 1000000}}
+                style={{zIndex: 1000000, maxWidth: 216, overflow: 'hidden', textOverflow: 'ellipsis'}}
                 onDelete={event => handleTagDelete(event, sortIndex)}
                 onClick={onChipClick ? event => onChipClick(event, option) : null}
                 label={getChipText(option)}
@@ -321,6 +351,7 @@ export default function AutocompleteVirtualized(props) {
         );
     });
     const multiple = props.multiple != null ? props.multiple : true;
+
     return <>
         <Autocomplete
             data-testid={props.testId}
@@ -334,15 +365,15 @@ export default function AutocompleteVirtualized(props) {
             autoHighlight={true}
             filterOptions={filterOptions}
             isOptionEqualToValue={getOptionSelected}
-            value={props.value}
+            value={value}
             filterSelectedOptions={true}
             getOptionLabel={getOptionLabel}
             groupBy={props.groupBy}
             ChipProps={{size: 'small'}}
             ListboxComponent={ListboxComponent}
             PopperComponent={StyledPopper}
-            options={props.options}
-            onChange={props.onChange}
+            options={options}
+            onChange={onChange}
             renderTags={(value, getTagProps) =>
                 null
             }
@@ -350,7 +381,14 @@ export default function AutocompleteVirtualized(props) {
                 <TextField {...params} label={props.label} helperText={props.helperText} fullWidth={true}
                            margin={"dense"}/>
             )}
-            renderGroup={(params) => params}
+            renderGroup={(params) => {
+                params.selectGroup = selectGroup;
+                params.inputValue = inputValueRef.current;
+                return params;
+            }}
+            onInputChange={(event, value, reason) => {
+                inputValueRef.current = value;
+            }}
             renderOption={(props, option, {inputValue}) => [props, option, inputValue]}
             onPaste={onPaste}
             onDrop={onDrop}
@@ -364,6 +402,7 @@ export default function AutocompleteVirtualized(props) {
             onSortEnd={onSortEnd}
             axis="xy" items={props.value}
         />}
-
     </>;
-}
+};
+
+
