@@ -233,10 +233,14 @@ export class DirectAccessDataset {
         return this.baseUrl + file;
     }
 
-    getJob(id, returnResults) {
+    getJob(id) {
         return fetch(this.baseUrl + 'uns/' + id + '.json').then(r => r.json());
     }
 
+    getJobParams(id) {
+        return Promise.resolve({});
+    }
+
     getJobs() {
         return Promise.resolve([]);
     }
