@@ -13,12 +13,12 @@ function JobResultOptions(props) {
 
 const mapStateToProps = state => {
         return {
-            jobResultId: state.jobResult,
+            jobResultId: state.jobResultId,
             jobResults: state.jobResults
         };
     }
 ;
-const mapDispatchToProps = (dispatch, ownProps) => {
+const mapDispatchToProps = (dispatch) => {
         return {};
     }
 ;
