@@ -53,9 +53,9 @@ def h5_dataset_force_sparse(request):
 
 
 @pytest.fixture(scope='module', autouse=True)
-def dataset_api(h5_dataset_backed, h5_dataset_force_sparse):
+def dataset_api(h5_dataset_backed):
     dataset_api = DatasetAPI()
-    dataset_api.add(AnndataDataset(backed=h5_dataset_backed, force_sparse=h5_dataset_force_sparse))
+    dataset_api.add(AnndataDataset(backed=h5_dataset_backed))
     return dataset_api
 
 
